#
# LECA Bash library
#
# The LECA bash library provides a set of function used for helping
# development of bash script mainly to write job script on the luke
# cluster
#
# The files from the LECA Bash library must be sourced from your main script
#
#
# Provides functions to facilitate downloading of files
# =====================================================
#
# Provided function:
#
#    - download <URL> [USERNAME] [PASSWORD]
#      Downloads the file specified by URL.
#      If specifed the second (USERNAME) and third (PASSWORD) parametters allow
#      to indicate a login and password for a protected connection
#
#    - downloadandcheck <URL> <MD5URL> <DESTINATION> [USERNAME] [PASSWORD]
#      Downloads the file specified by URL and check file integrity.
#      The MD5 checksum file URL is indicated using the MD5URL parametters.
#      You must specified a destination filename to store the downloaded data
#      using the DESTINATION parametter. The MD5 file is downloaded under the
#      name DESTINATION.md5
#      If specifed the fourth (USERNAME) and fifth (PASSWORD) parametters allow
#      to indicate a login and password for a protected connection
#
#      On error the function has a return status set to 1, 0 otherwise.
#
#    - setmaxretry <MAXRETRY>
#      If downloading failed the downloadandcheck function can retry to download
#      the file several times before a complete failure. This function allows to
#      define the maximum of time the downloadandcheck tries to redownload before
#      exiting in error
#
#

include logging
include ifs

logdebug "Load download package"

if ! which curl; then
	if module 2> /dev/null; then
	  module load curl
	  module load openssl
	else
	  logerror "No curl command available"
	fi
fi

if ! which md5; then
	if ! which md5sum; then
		logerror "No md5 commands available"
	else
		LECABASHLIB_MD5SUM=md5sum
	fi
else
	LECABASHLIB_MD5SUM=md5
fi

CURL="curl -L"
LECA_MAX_RETRY=3

function download() {
  pushifs
  
  logdebug "Function called with $# arguments"
  logdebug "$@"
  local URL="${1}"
  shift

  local LINK="${CURL} ${URL}"

  if [[ ! -z $1 && -z $2 ]]; then
    logdebug "Download with username ${1}"
    local LINK="${CURL} --user ${1} ${URL}"
    shift
  fi

  if [[ ! -z $1 && ! -z $2 ]]; then
    logdebug "Download with username ${1} and password ${2}"
    local LINK="${CURL} --user ${1}:${2} ${URL}"
    shift
    shift
  fi

  loginfo "Downloading URL : ${URL}"
  ${LINK}
  loginfo "URL : ${URL} -> Done."
  
  popifs
}

function setmaxretry() {
  LECA_MAX_RETRY="$1"
}

function downloadandcheck() {
  local URL="${1}"
  shift

  local MD5="${1}"
  shift

  local TARGETFILE="${1}"
  shift

  if [[ ! -z "${1}" ]]; then
    local DOWLOADUSER="${1}"
    shift
  fi

  if [[ ! -z "${1}" ]]; then
    local PASSWORD="${1}"
    shift
  fi

  local RETRY=0
  local MD5_BAD=1

  while (( MD5_BAD > 0 )) && (( RETRY < LECA_MAX_RETRY )); do
    download "${URL}" "${DOWLOADUSER}" "${PASSWORD}" > "${TARGETFILE}"
    download "${MD5}" "${DOWLOADUSER}" "${PASSWORD}" > "${TARGETFILE}.md5"

    MD5LOCAL=$(${LECABASHLIB_MD5SUM} "${TARGETFILE}" | awk '{print $1}')
    MD5ORIGI=$(awk '{print $1}' "${TARGETFILE}.md5")

    if [[ "$MD5LOCAL" == "$MD5ORIGI" ]]; then
      MD5_BAD=0
    else
      MD5_BAD=1
    fi

    RETRY=$((RETRY+1))

    if (( MD5_BAD > 0 )); then
      logwarning "The two MD5 are different : $MD5LOCAL and $MD5ORIGI"
    fi
  done

    if (( MD5_BAD > 0 )); then
      logerror "The two MD5 are different after $RETRY retries: "
      logerror "  $MD5LOCAL and $MD5ORIGI"
      return 1
    else
      loginfo "${TARGETFILE} downloaded"
      loginfo "   and checked with MD5 $MD5ORIGI"
      return 0
    fi
}
